@@ -1,8 +1,11 @@
 """Base test class for DNS authenticators."""
-import typing
+from typing import Any
+from typing import Mapping
+from typing import TYPE_CHECKING
 
 import configobj
 import josepy as jose
+from unittest import mock
 
 from acme import challenges
 from certbot import achallenges
@@ -11,19 +14,12 @@ from certbot.plugins.dns_common import DNSAuthenticator
 from certbot.tests import acme_util
 from certbot.tests import util as test_util
 
-if typing.TYPE_CHECKING:
+if TYPE_CHECKING:
     from typing_extensions import Protocol
 else:
     Protocol = object
 
 
-
-try:
-    import mock
-except ImportError:  # pragma: no cover
-    from unittest import mock  # type: ignore
-
-
 DOMAIN = 'example.com'
 KEY = jose.JWKRSA.load(test_util.load_vector("rsa512_key.pem"))
 
@@ -32,19 +28,17 @@ class _AuthenticatorCallableTestCase(Protocol):
     """Protocol describing a TestCase able to call a real DNSAuthenticator instance."""
     auth: DNSAuthenticator
 
-    def assertTrue(self, *unused_args) -> None:
+    def assertTrue(self, *unused_args: Any) -> None:
         """
         See
         https://docs.python.org/3/library/unittest.html#unittest.TestCase.assertTrue
         """
-        ...
 
-    def assertEqual(self, *unused_args) -> None:
+    def assertEqual(self, *unused_args: Any) -> None:
         """
         See
         https://docs.python.org/3/library/unittest.html#unittest.TestCase.assertEqual
         """
-        ...
 
 
 class BaseAuthenticatorTest:
@@ -59,20 +53,20 @@ class BaseAuthenticatorTest:
     achall = achallenges.KeyAuthorizationAnnotatedChallenge(
         challb=acme_util.DNS01, domain=DOMAIN, account_key=KEY)
 
-    def test_more_info(self: _AuthenticatorCallableTestCase):
+    def test_more_info(self: _AuthenticatorCallableTestCase) -> None:
         self.assertTrue(isinstance(self.auth.more_info(), str))  # pylint: disable=no-member
 
-    def test_get_chall_pref(self: _AuthenticatorCallableTestCase):
-        self.assertEqual(self.auth.get_chall_pref(None), [challenges.DNS01])  # pylint: disable=no-member
+    def test_get_chall_pref(self: _AuthenticatorCallableTestCase) -> None:
+        self.assertEqual(self.auth.get_chall_pref("example.org"), [challenges.DNS01])  # pylint: disable=no-member
 
-    def test_parser_arguments(self: _AuthenticatorCallableTestCase):
+    def test_parser_arguments(self: _AuthenticatorCallableTestCase) -> None:
         m = mock.MagicMock()
         self.auth.add_parser_arguments(m)  # pylint: disable=no-member
 
         m.assert_any_call('propagation-seconds', type=int, default=mock.ANY, help=mock.ANY)
 
 
-def write(values, path):
+def write(values: Mapping[str, Any], path: str) -> None:
     """Write the specified values to a config file.
 
     :param dict values: A map of values to write.
