@@ -2,21 +2,26 @@
 from importlib import reload as reload_module
 import io
 import logging
-from multiprocessing import Event
-from multiprocessing import Process
+import multiprocessing
+from multiprocessing import synchronize
 import shutil
 import sys
 import tempfile
+from typing import Any
+from typing import Callable
+from typing import Union
+from typing import cast
+from typing import IO
 from typing import Iterable
 from typing import List
 from typing import Optional
 import unittest
-import warnings
+from unittest import mock
 
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import serialization
 import josepy as jose
-import OpenSSL
+from OpenSSL import crypto
 import pkg_resources
 
 from certbot import configuration
@@ -30,35 +35,22 @@ from certbot.compat import os
 from certbot.display import util as display_util
 from certbot.plugins import common
 
-try:
-    # When we remove this deprecated import, we should also remove the
-    # "external-mock" test environment and the mock dependency listed in
-    # tools/pinning/pyproject.toml.
-    import mock
-    warnings.warn(
-        "The external mock module is being used for backwards compatibility "
-        "since it is available, however, future versions of Certbot's tests will "
-        "use unittest.mock. Be sure to update your code accordingly.",
-        PendingDeprecationWarning
-    )
-except ImportError:  # pragma: no cover
-    from unittest import mock  # type: ignore
-
 
 class DummyInstaller(common.Installer):
     """Dummy installer plugin for test purpose."""
     def get_all_names(self) -> Iterable[str]:
-        pass
+        return []
 
     def deploy_cert(self, domain: str, cert_path: str, key_path: str, chain_path: str,
                     fullchain_path: str) -> None:
         pass
 
-    def enhance(self, domain: str, enhancement: str, options: Optional[List[str]] = None) -> None:
+    def enhance(self, domain: str, enhancement: str,
+                options: Optional[Union[List[str], str]] = None) -> None:
         pass
 
     def supported_enhancements(self) -> List[str]:
-        pass
+        return []
 
     def save(self, title: Optional[str] = None, temporary: bool = False) -> None:
         pass
@@ -70,23 +62,23 @@ class DummyInstaller(common.Installer):
         pass
 
     @classmethod
-    def add_parser_arguments(cls, add):
+    def add_parser_arguments(cls, add: Callable[..., None]) -> None:
         pass
 
     def prepare(self) -> None:
         pass
 
     def more_info(self) -> str:
-        pass
+        return ""
 
 
-def vector_path(*names):
+def vector_path(*names: str) -> str:
     """Path to a test vector."""
     return pkg_resources.resource_filename(
         __name__, os.path.join('testdata', *names))
 
 
-def load_vector(*names):
+def load_vector(*names: str) -> bytes:
     """Load contents of a test vector."""
     # luckily, resource_string opens file in binary mode
     data = pkg_resources.resource_string(
@@ -100,7 +92,7 @@ def load_vector(*names):
         return data
 
 
-def _guess_loader(filename, loader_pem, loader_der):
+def _guess_loader(filename: str, loader_pem: int, loader_der: int) -> int:
     _, ext = os.path.splitext(filename)
     if ext.lower() == '.pem':
         return loader_pem
@@ -109,41 +101,45 @@ def _guess_loader(filename, loader_pem, loader_der):
     raise ValueError("Loader could not be recognized based on extension")  # pragma: no cover
 
 
-def load_cert(*names):
+def load_cert(*names: str) -> crypto.X509:
     """Load certificate."""
     loader = _guess_loader(
-        names[-1], OpenSSL.crypto.FILETYPE_PEM, OpenSSL.crypto.FILETYPE_ASN1)
-    return OpenSSL.crypto.load_certificate(loader, load_vector(*names))
+        names[-1], crypto.FILETYPE_PEM, crypto.FILETYPE_ASN1)
+    return crypto.load_certificate(loader, load_vector(*names))
 
 
-def load_csr(*names):
+def load_csr(*names: str) -> crypto.X509Req:
     """Load certificate request."""
     loader = _guess_loader(
-        names[-1], OpenSSL.crypto.FILETYPE_PEM, OpenSSL.crypto.FILETYPE_ASN1)
-    return OpenSSL.crypto.load_certificate_request(loader, load_vector(*names))
+        names[-1], crypto.FILETYPE_PEM, crypto.FILETYPE_ASN1)
+    return crypto.load_certificate_request(loader, load_vector(*names))
 
 
-def load_comparable_csr(*names):
+def load_comparable_csr(*names: str) -> jose.ComparableX509:
     """Load ComparableX509 certificate request."""
     return jose.ComparableX509(load_csr(*names))
 
 
-def load_rsa_private_key(*names):
+def load_rsa_private_key(*names: str) -> jose.ComparableRSAKey:
     """Load RSA private key."""
-    loader = _guess_loader(names[-1], serialization.load_pem_private_key,
-                           serialization.load_der_private_key)
-    return jose.ComparableRSAKey(loader(
+    loader = _guess_loader(names[-1], crypto.FILETYPE_PEM, crypto.FILETYPE_ASN1)
+    loader_fn: Callable[..., Any]
+    if loader == crypto.FILETYPE_PEM:
+        loader_fn = serialization.load_pem_private_key
+    else:
+        loader_fn = serialization.load_der_private_key
+    return jose.ComparableRSAKey(loader_fn(
         load_vector(*names), password=None, backend=default_backend()))
 
 
-def load_pyopenssl_private_key(*names):
+def load_pyopenssl_private_key(*names: str) -> crypto.PKey:
     """Load pyOpenSSL private key."""
     loader = _guess_loader(
-        names[-1], OpenSSL.crypto.FILETYPE_PEM, OpenSSL.crypto.FILETYPE_ASN1)
-    return OpenSSL.crypto.load_privatekey(loader, load_vector(*names))
+        names[-1], crypto.FILETYPE_PEM, crypto.FILETYPE_ASN1)
+    return crypto.load_privatekey(loader, load_vector(*names))
 
 
-def make_lineage(config_dir, testfile, ec=False):
+def make_lineage(config_dir: str, testfile: str, ec: bool = True) -> str:
     """Creates a lineage defined by testfile.
 
     This creates the archive, live, and renewal directories if
@@ -151,6 +147,7 @@ def make_lineage(config_dir, testfile, ec=False):
 
     :param str config_dir: path to the configuration directory
     :param str testfile: configuration file to base the lineage on
+    :param bool ec: True if we generate the lineage with an ECDSA key
 
     :returns: path to the renewal conf file for the created lineage
     :rtype: str
@@ -187,81 +184,43 @@ def make_lineage(config_dir, testfile, ec=False):
     return conf_path
 
 
-def patch_get_utility(target='zope.component.getUtility'):
-    """Deprecated, patch certbot.display.util directly or use patch_display_util instead.
-
-    :param str target: path to patch
-
-    :returns: mock zope.component.getUtility
-    :rtype: mock.MagicMock
-
-    """
-    warnings.warn('Decorator certbot.tests.util.patch_get_utility is deprecated. You should now '
-                  'patch certbot.display.util yourself directly or use '
-                  'certbot.tests.util.patch_display_util as a temporary workaround.')
-    return mock.patch(target, new_callable=_create_display_util_mock)
-
-
-def patch_get_utility_with_stdout(target='zope.component.getUtility',
-                                  stdout=None):
-    """Deprecated, patch certbot.display.util directly
-    or use patch_display_util_with_stdout instead.
-
-    :param str target: path to patch
-    :param object stdout: object to write standard output to; it is
-        expected to have a `write` method
-
-    :returns: mock zope.component.getUtility
-    :rtype: mock.MagicMock
-
-    """
-    warnings.warn('Decorator certbot.tests.util.patch_get_utility_with_stdout is deprecated. You '
-                  'should now patch certbot.display.util yourself directly or use '
-                  'use certbot.tests.util.patch_display_util_with_stdout as a temporary '
-                  'workaround.')
-    stdout = stdout if stdout else io.StringIO()
-    freezable_mock = _create_display_util_mock_with_stdout(stdout)
-    return mock.patch(target, new=freezable_mock)
-
-
-def patch_display_util():
+def patch_display_util() -> mock.MagicMock:
     """Patch certbot.display.util to use a special mock display utility.
 
     The mock display utility works like a regular mock object, except it also
     also asserts that methods are called with valid arguments.
 
-    The mock created by this patch mocks out Certbot internals so this can be
-    used like the old patch_get_utility function. That is, the mock object will
-    be called by the certbot.display.util functions and the mock returned by
-    that call will be used as the display utility. This was done to simplify
-    the transition from zope.component and mocking certbot.display.util
-    functions directly in test code should be preferred over using this
-    function in the future.
+    The mock created by this patch mocks out Certbot internals. That is, the
+    mock object will be called by the certbot.display.util functions and the
+    mock returned by that call will be used as the display utility. This was
+    done to simplify the transition from zope.component and mocking
+    certbot.display.util functions directly in test code should be preferred
+    over using this function in the future.
 
     See https://github.com/certbot/certbot/issues/8948
 
     :returns: patch on the function used internally by certbot.display.util to
         get a display utility instance
-    :rtype: unittest.mock._patch
+    :rtype: mock.MagicMock
 
     """
-    return mock.patch('certbot._internal.display.obj.get_display',
-                      new_callable=_create_display_util_mock)
+    return cast(mock.MagicMock, mock.patch('certbot._internal.display.obj.get_display',
+                                           new_callable=_create_display_util_mock))
 
 
-def patch_display_util_with_stdout(stdout=None):
+def patch_display_util_with_stdout(
+        stdout: Optional[IO] = None) -> mock.MagicMock:
     """Patch certbot.display.util to use a special mock display utility.
 
     The mock display utility works like a regular mock object, except it also
     asserts that methods are called with valid arguments.
 
-    The mock created by this patch mocks out Certbot internals so this can be
-    used like the old patch_get_utility function. That is, the mock object will
-    be called by the certbot.display.util functions and the mock returned by
-    that call will be used as the display utility. This was done to simplify
-    the transition from zope.component and mocking certbot.display.util
-    functions directly in test code should be preferred over using this
-    function in the future.
+    The mock created by this patch mocks out Certbot internals. That is, the
+    mock object will be called by the certbot.display.util functions and the
+    mock returned by that call will be used as the display utility. This was
+    done to simplify the transition from zope.component and mocking
+    certbot.display.util functions directly in test code should be preferred
+    over using this function in the future.
 
     See https://github.com/certbot/certbot/issues/8948
 
@@ -272,13 +231,13 @@ def patch_display_util_with_stdout(stdout=None):
         expected to have a `write` method
     :returns: patch on the function used internally by certbot.display.util to
         get a display utility instance
-    :rtype: unittest.mock._patch
+    :rtype: mock.MagicMock
 
     """
     stdout = stdout if stdout else io.StringIO()
 
-    return mock.patch('certbot._internal.display.obj.get_display',
-                      new=_create_display_util_mock_with_stdout(stdout))
+    return cast(mock.MagicMock, mock.patch('certbot._internal.display.obj.get_display',
+                                           new=_create_display_util_mock_with_stdout(stdout)))
 
 
 class FreezableMock:
@@ -294,7 +253,8 @@ class FreezableMock:
     value of func is ignored.
 
     """
-    def __init__(self, frozen=False, func=None, return_value=mock.sentinel.DEFAULT):
+    def __init__(self, frozen: bool = False, func: Optional[Callable[..., Any]] = None,
+                 return_value: Any = mock.sentinel.DEFAULT) -> None:
         self._frozen_set = set() if frozen else {'freeze', }
         self._func = func
         self._mock = mock.MagicMock()
@@ -302,16 +262,16 @@ class FreezableMock:
             self.return_value = return_value
         self._frozen = frozen
 
-    def freeze(self):
+    def freeze(self) -> None:
         """Freeze object preventing further changes."""
         self._frozen = True
 
-    def __call__(self, *args, **kwargs):
+    def __call__(self, *args: Any, **kwargs: Any) -> mock.MagicMock:
         if self._func is not None:
             self._func(*args, **kwargs)
         return self._mock(*args, **kwargs)
 
-    def __getattribute__(self, name):
+    def __getattribute__(self, name: str) -> Any:
         if name == '_frozen':
             try:
                 return object.__getattribute__(self, name)
@@ -324,7 +284,7 @@ class FreezableMock:
         else:
             return getattr(object.__getattribute__(self, '_mock'), name)
 
-    def __setattr__(self, name, value):
+    def __setattr__(self, name: str, value: Any) -> None:
         """ Before it is frozen, attributes are set on the FreezableMock
         instance and added to the _frozen_set. Attributes in the _frozen_set
         cannot be changed after the FreezableMock is frozen. In this case,
@@ -349,7 +309,7 @@ class FreezableMock:
         return object.__setattr__(self, name, value)
 
 
-def _create_display_util_mock():
+def _create_display_util_mock() -> FreezableMock:
     display = FreezableMock()
     # Use pylint code for disable to keep on single line under line length limit
     method_list = [func for func in dir(display_obj.FileDisplay)
@@ -363,14 +323,14 @@ def _create_display_util_mock():
     return FreezableMock(frozen=True, return_value=display)
 
 
-def _create_display_util_mock_with_stdout(stdout):
-    def _write_msg(message, *unused_args, **unused_kwargs):
+def _create_display_util_mock_with_stdout(stdout: IO) -> FreezableMock:
+    def _write_msg(message: str, *unused_args: Any, **unused_kwargs: Any) -> None:
         """Write to message to stdout.
         """
         if message:
             stdout.write(message)
 
-    def mock_method(*args, **kwargs):
+    def mock_method(*args: Any, **kwargs: Any) -> None:
         """
         Mock function for display utility methods.
         """
@@ -394,13 +354,14 @@ def _create_display_util_mock_with_stdout(stdout):
     return FreezableMock(frozen=True, return_value=display)
 
 
-def _assert_valid_call(*args, **kwargs):
+def _assert_valid_call(*args: Any, **kwargs: Any) -> None:
     assert_args = [args[0] if args else kwargs['message']]
 
-    assert_kwargs = {}
-    assert_kwargs['default'] = kwargs.get('default', None)
-    assert_kwargs['cli_flag'] = kwargs.get('cli_flag', None)
-    assert_kwargs['force_interactive'] = kwargs.get('force_interactive', False)
+    assert_kwargs = {
+        'default': kwargs.get('default', None),
+        'cli_flag': kwargs.get('cli_flag', None),
+        'force_interactive': kwargs.get('force_interactive', False),
+    }
 
     display_util.assert_valid_call(*assert_args, **assert_kwargs)
 
@@ -408,11 +369,11 @@ def _assert_valid_call(*args, **kwargs):
 class TempDirTestCase(unittest.TestCase):
     """Base test class which sets up and tears down a temporary directory"""
 
-    def setUp(self):
+    def setUp(self) -> None:
         """Execute before test"""
         self.tempdir = tempfile.mkdtemp()
 
-    def tearDown(self):
+    def tearDown(self) -> None:
         """Execute after test"""
         # Cleanup opened resources after a test. This is usually done through atexit handlers in
         # Certbot, but during tests, atexit will not run registered functions before tearDown is
@@ -429,7 +390,7 @@ class TempDirTestCase(unittest.TestCase):
 
 class ConfigTestCase(TempDirTestCase):
     """Test class which sets up a NamespaceConfig object."""
-    def setUp(self):
+    def setUp(self) -> None:
         super().setUp()
         self.config = configuration.NamespaceConfig(
             mock.MagicMock(**constants.CLI_DEFAULTS)
@@ -444,7 +405,7 @@ class ConfigTestCase(TempDirTestCase):
         self.config.namespace.server = "https://example.com"
 
 
-def _handle_lock(event_in, event_out, path):
+def _handle_lock(event_in: synchronize.Event, event_out: synchronize.Event, path: str) -> None:
     """
     Acquire a file lock on given path, then wait to release it. This worker is coordinated
     using events to signal when the lock should be acquired and released.
@@ -463,7 +424,7 @@ def _handle_lock(event_in, event_out, path):
         my_lock.release()
 
 
-def lock_and_call(callback, path_to_lock):
+def lock_and_call(callback: Callable[[], Any], path_to_lock: str) -> None:
     """
     Grab a lock on path_to_lock from a foreign process then execute the callback.
     :param callable callback: object to call after acquiring the lock
@@ -472,9 +433,10 @@ def lock_and_call(callback, path_to_lock):
     # Reload certbot.util module to reset internal _LOCKS dictionary.
     reload_module(util)
 
-    emit_event = Event()
-    receive_event = Event()
-    process = Process(target=_handle_lock, args=(emit_event, receive_event, path_to_lock))
+    emit_event = multiprocessing.Event()
+    receive_event = multiprocessing.Event()
+    process = multiprocessing.Process(target=_handle_lock,
+                                      args=(emit_event, receive_event, path_to_lock))
     process.start()
 
     # Wait confirmation that lock is acquired
@@ -489,15 +451,15 @@ def lock_and_call(callback, path_to_lock):
     assert process.exitcode == 0
 
 
-def skip_on_windows(reason):
+def skip_on_windows(reason: str) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
     """Decorator to skip permanently a test on Windows. A reason is required."""
-    def wrapper(function):
+    def wrapper(function: Callable[..., Any]) -> Callable[..., Any]:
         """Wrapped version"""
         return unittest.skipIf(sys.platform == 'win32', reason)(function)
     return wrapper
 
 
-def temp_join(path):
+def temp_join(path: str) -> str:
     """
     Return the given path joined to the tempdir path for the current platform
     Eg.: 'cert' => /tmp/cert (Linux) or 'C:\\Users\\currentuser\\AppData\\Temp\\cert' (Windows)
