@@ -3,10 +3,8 @@ import functools
 import logging
 import unittest
 
-try:
-    import mock
-except ImportError:  # pragma: no cover
-    from unittest import mock
+from josepy import b64encode
+from unittest import mock
 
 from acme import challenges
 from acme import client as acme_client
@@ -32,7 +30,7 @@ class ChallengeFactoryTest(unittest.TestCase):
 
         self.authzr = acme_util.gen_authzr(
             messages.STATUS_PENDING, "test", acme_util.CHALLENGES,
-            [messages.STATUS_PENDING] * 6, False)
+            [messages.STATUS_PENDING] * 6)
 
     def test_all(self):
         achalls = self.handler._challenge_factory(
@@ -69,18 +67,16 @@ class HandleAuthorizationsTest(unittest.TestCase):
 
         self.mock_display = mock.Mock()
         self.mock_config = mock.Mock(debug_challenges=False)
-        with mock.patch("zope.component.provideUtility"):
-            display_obj.set_display(self.mock_display)
+        display_obj.set_display(self.mock_display)
 
-        self.mock_auth = mock.MagicMock(name="ApacheConfigurator")
+        self.mock_auth = mock.MagicMock(name="Authenticator")
 
         self.mock_auth.get_chall_pref.return_value = [challenges.HTTP01]
 
         self.mock_auth.perform.side_effect = gen_auth_resp
 
-        self.mock_account = mock.Mock(key=util.Key("file_path", "PEM"))
+        self.mock_account = mock.MagicMock()
         self.mock_net = mock.MagicMock(spec=acme_client.ClientV2)
-        self.mock_net.acme_version = 1
         self.mock_net.retry_after.side_effect = acme_client.ClientV2.retry_after
 
         self.handler = AuthHandler(
@@ -91,8 +87,8 @@ class HandleAuthorizationsTest(unittest.TestCase):
     def tearDown(self):
         logging.disable(logging.NOTSET)
 
-    def _test_name1_http_01_1_common(self, combos):
-        authzr = gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES, combos=combos)
+    def _test_name1_http_01_1_common(self):
+        authzr = gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES)
         mock_order = mock.MagicMock(authorizations=[authzr])
 
         self.mock_net.poll.side_effect = _gen_mock_on_poll(retry=1, wait_value=30)
@@ -116,39 +112,14 @@ class HandleAuthorizationsTest(unittest.TestCase):
 
             self.assertEqual(len(authzr), 1)
 
-    def test_name1_http_01_1_acme_1(self):
-        self._test_name1_http_01_1_common(combos=True)
-
     def test_name1_http_01_1_acme_2(self):
-        self.mock_net.acme_version = 2
-        self._test_name1_http_01_1_common(combos=False)
-
-    def test_name1_http_01_1_dns_1_acme_1(self):
-        self.mock_net.poll.side_effect = _gen_mock_on_poll()
-        self.mock_auth.get_chall_pref.return_value.append(challenges.DNS01)
-
-        authzr = gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES, combos=False)
-        mock_order = mock.MagicMock(authorizations=[authzr])
-        authzr = self.handler.handle_authorizations(mock_order, self.mock_config)
-
-        self.assertEqual(self.mock_net.answer_challenge.call_count, 2)
-
-        self.assertEqual(self.mock_net.poll.call_count, 1)
-
-        self.assertEqual(self.mock_auth.cleanup.call_count, 1)
-        # Test if list first element is http-01, use typ because it is an achall
-        for achall in self.mock_auth.cleanup.call_args[0][0]:
-            self.assertIn(achall.typ, ["http-01", "dns-01"])
-
-        # Length of authorizations list
-        self.assertEqual(len(authzr), 1)
+        self._test_name1_http_01_1_common()
 
     def test_name1_http_01_1_dns_1_acme_2(self):
-        self.mock_net.acme_version = 2
         self.mock_net.poll.side_effect = _gen_mock_on_poll()
         self.mock_auth.get_chall_pref.return_value.append(challenges.DNS01)
 
-        authzr = gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES, combos=False)
+        authzr = gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES)
         mock_order = mock.MagicMock(authorizations=[authzr])
         authzr = self.handler.handle_authorizations(mock_order, self.mock_config)
 
@@ -164,7 +135,7 @@ class HandleAuthorizationsTest(unittest.TestCase):
         # Length of authorizations list
         self.assertEqual(len(authzr), 1)
 
-    def _test_name3_http_01_3_common(self, combos):
+    def test_name3_http_01_3_common_acme_2(self):
         authzrs = [gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES),
                    gen_dom_authzr(domain="1", challs=acme_util.CHALLENGES),
                    gen_dom_authzr(domain="2", challs=acme_util.CHALLENGES)]
@@ -182,24 +153,57 @@ class HandleAuthorizationsTest(unittest.TestCase):
 
         self.assertEqual(len(authzr), 3)
 
-    def test_name3_http_01_3_common_acme_1(self):
-        self._test_name3_http_01_3_common(combos=True)
-
-    def test_name3_http_01_3_common_acme_2(self):
-        self.mock_net.acme_version = 2
-        self._test_name3_http_01_3_common(combos=False)
-
     def test_debug_challenges(self):
-        config = mock.Mock(debug_challenges=True)
+        config = mock.Mock(debug_challenges=True, verbose_count=0)
         authzrs = [gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES)]
         mock_order = mock.MagicMock(authorizations=authzrs)
 
+        account_key_thumbprint = b"foobarbaz"
+        self.mock_account.key.thumbprint.return_value = account_key_thumbprint
+
         self.mock_net.poll.side_effect = _gen_mock_on_poll()
 
         self.handler.handle_authorizations(mock_order, config)
 
         self.assertEqual(self.mock_net.answer_challenge.call_count, 1)
         self.assertEqual(self.mock_display.notification.call_count, 1)
+        self.assertIn('Pass "-v" for more info',
+                      self.mock_display.notification.call_args[0][0])
+        self.assertNotIn(f"http://{authzrs[0].body.identifier.value}/.well-known/acme-challenge/" +
+                         b64encode(authzrs[0].body.challenges[0].chall.token).decode(),
+                         self.mock_display.notification.call_args[0][0])
+        self.assertNotIn(b64encode(account_key_thumbprint).decode(),
+                      self.mock_display.notification.call_args[0][0])
+
+    def test_debug_challenges_verbose(self):
+        config = mock.Mock(debug_challenges=True, verbose_count=1)
+        authzrs = [gen_dom_authzr(domain="0", challs=[acme_util.HTTP01]),
+                   gen_dom_authzr(domain="1", challs=[acme_util.DNS01])]
+        mock_order = mock.MagicMock(authorizations=authzrs)
+
+        account_key_thumbprint = b"foobarbaz"
+        self.mock_account.key.thumbprint.return_value = account_key_thumbprint
+
+        self.mock_net.poll.side_effect = _gen_mock_on_poll()
+
+        self.mock_auth.get_chall_pref.return_value = [challenges.HTTP01,
+                                                      challenges.DNS01]
+
+        self.handler.handle_authorizations(mock_order, config)
+
+        self.assertEqual(self.mock_net.answer_challenge.call_count, 2)
+        self.assertEqual(self.mock_display.notification.call_count, 1)
+        self.assertNotIn('Pass "-v" for more info',
+                         self.mock_display.notification.call_args[0][0])
+        self.assertIn(f"http://{authzrs[0].body.identifier.value}/.well-known/acme-challenge/" +
+                      b64encode(authzrs[0].body.challenges[0].chall.token).decode(),
+                      self.mock_display.notification.call_args[0][0])
+        self.assertIn(b64encode(account_key_thumbprint).decode(),
+                      self.mock_display.notification.call_args[0][0])
+        self.assertIn(f"_acme-challenge.{authzrs[1].body.identifier.value}",
+                      self.mock_display.notification.call_args[0][0])
+        self.assertIn(authzrs[1].body.challenges[0].validation(self.mock_account.key),
+                      self.mock_display.notification.call_args[0][0])
 
     def test_perform_failure(self):
         authzrs = [gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES)]
@@ -228,8 +232,8 @@ class HandleAuthorizationsTest(unittest.TestCase):
         self.assertRaises(errors.AuthorizationError, self.handler.handle_authorizations,
                           mock_order, self.mock_config)
 
-    def _test_preferred_challenge_choice_common(self, combos):
-        authzrs = [gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES, combos=combos)]
+    def test_preferred_challenge_choice_common_acme_2(self):
+        authzrs = [gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES)]
         mock_order = mock.MagicMock(authorizations=authzrs)
 
         self.mock_auth.get_chall_pref.return_value.append(challenges.HTTP01)
@@ -244,28 +248,14 @@ class HandleAuthorizationsTest(unittest.TestCase):
         self.assertEqual(
             self.mock_auth.cleanup.call_args[0][0][0].typ, "http-01")
 
-    def test_preferred_challenge_choice_common_acme_1(self):
-        self._test_preferred_challenge_choice_common(combos=True)
-
-    def test_preferred_challenge_choice_common_acme_2(self):
-        self.mock_net.acme_version = 2
-        self._test_preferred_challenge_choice_common(combos=False)
-
-    def _test_preferred_challenges_not_supported_common(self, combos):
-        authzrs = [gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES, combos=combos)]
+    def test_preferred_challenges_not_supported_acme_2(self):
+        authzrs = [gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES)]
         mock_order = mock.MagicMock(authorizations=authzrs)
         self.handler.pref_challs.append(challenges.DNS01.typ)
         self.assertRaises(
             errors.AuthorizationError, self.handler.handle_authorizations,
             mock_order, self.mock_config)
 
-    def test_preferred_challenges_not_supported_acme_1(self):
-        self._test_preferred_challenges_not_supported_common(combos=True)
-
-    def test_preferred_challenges_not_supported_acme_2(self):
-        self.mock_net.acme_version = 2
-        self._test_preferred_challenges_not_supported_common(combos=False)
-
     def test_dns_only_challenge_not_supported(self):
         authzrs = [gen_dom_authzr(domain="0", challs=[acme_util.DNS01])]
         mock_order = mock.MagicMock(authorizations=authzrs)
@@ -276,7 +266,7 @@ class HandleAuthorizationsTest(unittest.TestCase):
     def test_perform_error(self):
         self.mock_auth.perform.side_effect = errors.AuthorizationError
 
-        authzr = gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES, combos=True)
+        authzr = gen_dom_authzr(domain="0", challs=acme_util.CHALLENGES)
         mock_order = mock.MagicMock(authorizations=[authzr])
         self.assertRaises(errors.AuthorizationError, self.handler.handle_authorizations,
                           mock_order, self.mock_config)
@@ -351,7 +341,7 @@ class HandleAuthorizationsTest(unittest.TestCase):
         authzr = acme_util.gen_authzr(
                 messages.STATUS_PENDING, "0",
                 [acme_util.DNS01],
-                [messages.STATUS_PENDING], False)
+                [messages.STATUS_PENDING])
         mock_order = mock.MagicMock(authorizations=[authzr])
         self.assertRaises(
             errors.AuthorizationError, self.handler.handle_authorizations,
@@ -363,7 +353,7 @@ class HandleAuthorizationsTest(unittest.TestCase):
         authzr = acme_util.gen_authzr(
                 messages.STATUS_VALID, "0",
                 [acme_util.DNS01],
-                [messages.STATUS_VALID], False)
+                [messages.STATUS_VALID])
         mock_order = mock.MagicMock(authorizations=[authzr])
         self.handler.handle_authorizations(mock_order, self.mock_config)
 
@@ -385,7 +375,7 @@ class HandleAuthorizationsTest(unittest.TestCase):
                          ("is_valid_but_will_fail", messages.STATUS_VALID)]
 
         to_deactivate = [acme_util.gen_authzr(a[1], a[0], [acme_util.HTTP01],
-                         [a[1], False]) for a in to_deactivate]
+                         [a[1]]) for a in to_deactivate]
         orderr = mock.MagicMock(authorizations=to_deactivate)
 
         self.mock_net.deactivate_authorization.side_effect = _mock_deactivate
@@ -411,8 +401,7 @@ def _gen_mock_on_poll(status=messages.STATUS_VALID, retry=0, wait_value=1):
             effective_status,
             authzr.body.identifier.value,
             [challb.chall for challb in authzr.body.challenges],
-            [effective_status] * len(authzr.body.challenges),
-            authzr.body.combinations)
+            [effective_status] * len(authzr.body.challenges))
         return updated_azr, mock.MagicMock(headers={'Retry-After': str(wait_value)})
     return _mock
 
@@ -436,8 +425,6 @@ class ChallbToAchallTest(unittest.TestCase):
 class GenChallengePathTest(unittest.TestCase):
     """Tests for certbot._internal.auth_handler.gen_challenge_path.
 
-    .. todo:: Add more tests for dumb_path... depending on what we want to do.
-
     """
     def setUp(self):
         logging.disable(logging.FATAL)
@@ -446,34 +433,25 @@ class GenChallengePathTest(unittest.TestCase):
         logging.disable(logging.NOTSET)
 
     @classmethod
-    def _call(cls, challbs, preferences, combinations):
+    def _call(cls, challbs, preferences):
         from certbot._internal.auth_handler import gen_challenge_path
-        return gen_challenge_path(challbs, preferences, combinations)
+        return gen_challenge_path(challbs, preferences)
 
     def test_common_case(self):
         """Given DNS01 and HTTP01 with appropriate combos."""
         challbs = (acme_util.DNS01_P, acme_util.HTTP01_P)
         prefs = [challenges.DNS01, challenges.HTTP01]
-        combos = ((0,), (1,))
 
-        # Smart then trivial dumb path test
-        self.assertEqual(self._call(challbs, prefs, combos), (0,))
-        self.assertTrue(self._call(challbs, prefs, None))
-        # Rearrange order...
-        self.assertEqual(self._call(challbs[::-1], prefs, combos), (1,))
-        self.assertTrue(self._call(challbs[::-1], prefs, None))
+        self.assertEqual(self._call(challbs, prefs), (0,))
+        self.assertEqual(self._call(challbs[::-1], prefs), (1,))
 
     def test_not_supported(self):
-        challbs = (acme_util.DNS01_P, acme_util.HTTP01_P)
+        challbs = (acme_util.DNS01_P,)
         prefs = [challenges.HTTP01]
-        combos = ((0, 1),)
 
-        # smart path fails because no challs in perfs satisfies combos
+        # smart path fails because no challs in prefs satisfies combos
         self.assertRaises(
-            errors.AuthorizationError, self._call, challbs, prefs, combos)
-        # dumb path fails because all challbs are not supported
-        self.assertRaises(
-            errors.AuthorizationError, self._call, challbs, prefs, None)
+            errors.AuthorizationError, self._call, challbs, prefs)
 
 
 class ReportFailedAuthzrsTest(unittest.TestCase):
@@ -574,11 +552,11 @@ def gen_auth_resp(chall_list):
             for chall in chall_list]
 
 
-def gen_dom_authzr(domain, challs, combos=True):
+def gen_dom_authzr(domain, challs):
     """Generates new authzr for domains."""
     return acme_util.gen_authzr(
         messages.STATUS_PENDING, domain, challs,
-        [messages.STATUS_PENDING] * len(challs), combos)
+        [messages.STATUS_PENDING] * len(challs))
 
 
 if __name__ == "__main__":
