@@ -5,6 +5,7 @@ from importlib import reload as reload_module
 import io
 import tempfile
 import unittest
+from unittest import mock
 
 from acme import challenges
 from certbot import errors
@@ -16,11 +17,6 @@ from certbot.compat import os
 import certbot.tests.util as test_util
 from certbot.tests.util import TempDirTestCase
 
-try:
-    import mock
-except ImportError: # pragma: no cover
-    from unittest import mock
-
 
 PLUGINS = disco.PluginsRegistry.find_all()
 
@@ -85,7 +81,7 @@ class ParseTest(unittest.TestCase):
 
     @staticmethod
     def parse(*args, **kwargs):
-        """Mocks zope.component.getUtility and calls _unmocked_parse."""
+        """Mocks certbot._internal.display.obj.get_display and calls _unmocked_parse."""
         with test_util.patch_display_util():
             return ParseTest._unmocked_parse(*args, **kwargs)
 
