@@ -1,10 +1,6 @@
 """Tests for certbot.configuration."""
 import unittest
-
-try:
-    import mock
-except ImportError: # pragma: no cover
-    from unittest import mock
+from unittest import mock
 
 from certbot import errors
 from certbot._internal import constants
