@@ -2,6 +2,7 @@
 """Test certbot.display.ops."""
 import sys
 import unittest
+from unittest import mock
 
 import josepy as jose
 
@@ -15,11 +16,6 @@ from certbot.display import ops
 from certbot.display import util as display_util
 import certbot.tests.util as test_util
 
-try:
-    import mock
-except ImportError:  # pragma: no cover
-    from unittest import mock
-
 
 KEY = jose.JWKRSA.load(test_util.load_vector("rsa512_key.pem"))
 
@@ -474,7 +470,7 @@ class ChooseValuesTest(unittest.TestCase):
         result = self._call(items, None)
         self.assertEqual(result, [items[2]])
         self.assertIs(mock_util().checklist.called, True)
-        self.assertIsNone(mock_util().checklist.call_args[0][0])
+        self.assertEqual(mock_util().checklist.call_args[0][0], "")
 
     @test_util.patch_display_util()
     def test_choose_names_success_question(self, mock_util):
