@@ -1,11 +1,8 @@
 """Tests for certbot._internal.eff."""
 import datetime
 import unittest
+from unittest import mock
 
-try:
-    import mock
-except ImportError:  # pragma: no cover
-    from unittest import mock
 import josepy
 import pytz
 import requests
@@ -22,7 +19,7 @@ _KEY = josepy.JWKRSA.load(test_util.load_vector("rsa512_key.pem"))
 class SubscriptionTest(test_util.ConfigTestCase):
     """Abstract class for subscription tests."""
     def setUp(self):
-        super(SubscriptionTest, self).setUp()
+        super().setUp()
         self.account = account.Account(
             regr=messages.RegistrationResource(
                 uri=None, body=messages.Registration(),
@@ -42,7 +39,7 @@ class PrepareSubscriptionTest(SubscriptionTest):
         from certbot._internal.eff import prepare_subscription
         prepare_subscription(self.config, self.account)
 
-    @test_util.patch_get_utility()
+    @test_util.patch_display_util()
     @mock.patch("certbot._internal.eff.display_util.notify")
     def test_failure(self, mock_notify, mock_get_utility):
         self.config.email = None
@@ -50,24 +47,24 @@ class PrepareSubscriptionTest(SubscriptionTest):
         self._call()
         actual = mock_notify.call_args[0][0]
         expected_part = "because you didn't provide an e-mail address"
-        self.assertTrue(expected_part in actual)
+        self.assertIn(expected_part, actual)
         self.assertIsNone(self.account.meta.register_to_eff)
 
-    @test_util.patch_get_utility()
+    @test_util.patch_display_util()
     def test_will_not_subscribe_with_no_prompt(self, mock_get_utility):
         self.config.eff_email = False
         self._call()
         self._assert_no_get_utility_calls(mock_get_utility)
         self.assertIsNone(self.account.meta.register_to_eff)
 
-    @test_util.patch_get_utility()
+    @test_util.patch_display_util()
     def test_will_subscribe_with_no_prompt(self, mock_get_utility):
         self.config.eff_email = True
         self._call()
         self._assert_no_get_utility_calls(mock_get_utility)
         self.assertEqual(self.account.meta.register_to_eff, self.config.email)
 
-    @test_util.patch_get_utility()
+    @test_util.patch_display_util()
     def test_will_not_subscribe_with_prompt(self, mock_get_utility):
         mock_get_utility().yesno.return_value = False
         self._call()
@@ -75,7 +72,7 @@ class PrepareSubscriptionTest(SubscriptionTest):
         self._assert_correct_yesno_call(mock_get_utility)
         self.assertIsNone(self.account.meta.register_to_eff)
 
-    @test_util.patch_get_utility()
+    @test_util.patch_display_util()
     def test_will_subscribe_with_prompt(self, mock_get_utility):
         mock_get_utility().yesno.return_value = True
         self._call()
@@ -92,7 +89,7 @@ class PrepareSubscriptionTest(SubscriptionTest):
         call_args, call_kwargs = mock_get_utility().yesno.call_args
         actual = call_args[0]
         expected_part = 'Electronic Frontier Foundation'
-        self.assertTrue(expected_part in actual)
+        self.assertIn(expected_part, actual)
         self.assertFalse(call_kwargs.get('default', True))
 
 
@@ -105,7 +102,7 @@ class HandleSubscriptionTest(SubscriptionTest):
     @mock.patch('certbot._internal.eff.subscribe')
     def test_no_subscribe(self, mock_subscribe):
         self._call()
-        self.assertFalse(mock_subscribe.called)
+        self.assertIs(mock_subscribe.called, False)
 
     @mock.patch('certbot._internal.eff.subscribe')
     def test_subscribe(self, mock_subscribe):
@@ -140,7 +137,7 @@ class SubscribeTest(unittest.TestCase):
         self.assertEqual(call_args[0], constants.EFF_SUBSCRIBE_URI)
 
         data = call_kwargs.get('data')
-        self.assertFalse(data is None)
+        self.assertIsNotNone(data)
         self.assertEqual(data.get('email'), self.email)
 
     def test_bad_status(self):
@@ -148,7 +145,7 @@ class SubscribeTest(unittest.TestCase):
         self._call()
         actual = self._get_reported_message()
         expected_part = 'because your e-mail address appears to be invalid.'
-        self.assertTrue(expected_part in actual)
+        self.assertIn(expected_part, actual)
 
     def test_not_ok(self):
         self.response.ok = False
@@ -156,30 +153,30 @@ class SubscribeTest(unittest.TestCase):
         self._call()
         actual = self._get_reported_message()
         unexpected_part = 'because'
-        self.assertFalse(unexpected_part in actual)
+        self.assertNotIn(unexpected_part, actual)
 
     def test_response_not_json(self):
         self.response.json.side_effect = ValueError()
         self._call()
         actual = self._get_reported_message()
         expected_part = 'problem'
-        self.assertTrue(expected_part in actual)
+        self.assertIn(expected_part, actual)
 
     def test_response_json_missing_status_element(self):
         self.json.clear()
         self._call()
         actual = self._get_reported_message()
         expected_part = 'problem'
-        self.assertTrue(expected_part in actual)
+        self.assertIn(expected_part, actual)
 
     def _get_reported_message(self):
         self.assertTrue(self.mock_notify.called)
         return self.mock_notify.call_args[0][0]
 
-    @test_util.patch_get_utility()
+    @test_util.patch_display_util()
     def test_subscribe(self, mock_get_utility):
         self._call()
-        self.assertFalse(mock_get_utility.called)
+        self.assertIs(mock_get_utility.called, False)
 
 
 if __name__ == '__main__':
