@@ -1,10 +1,6 @@
 """Tests for certbot._internal.hooks."""
 import unittest
-
-try:
-    import mock
-except ImportError: # pragma: no cover
-    from unittest import mock
+from unittest import mock
 
 from certbot import errors
 from certbot import util
@@ -320,9 +316,10 @@ class RenewalHookTest(HookTest):
 
     def setUp(self):
         super().setUp()
-        self.vars_to_clear = set(
+        self.vars_to_clear = {
             var for var in ("RENEWED_DOMAINS", "RENEWED_LINEAGE",)
-            if var not in os.environ)
+            if var not in os.environ
+        }
 
     def tearDown(self):
         for var in self.vars_to_clear:
