@@ -6,6 +6,7 @@ import sys
 import time
 from typing import Optional
 import unittest
+from unittest import mock
 
 from acme import messages
 from certbot import errors
@@ -15,11 +16,6 @@ from certbot.compat import filesystem
 from certbot.compat import os
 from certbot.tests import util as test_util
 
-try:
-    import mock
-except ImportError: # pragma: no cover
-    from unittest import mock
-
 
 
 class PreArgParseSetupTest(unittest.TestCase):
