@@ -1,11 +1,7 @@
 """Tests for certbot.plugins.dns_common_lexicon."""
 
 import unittest
-
-try:
-    import mock
-except ImportError: # pragma: no cover
-    from unittest import mock
+from unittest import mock
 
 from certbot.plugins import dns_common_lexicon
 from certbot.plugins import dns_test_common_lexicon
