@@ -1,10 +1,6 @@
 """Tests for certbot.plugins.util."""
 import unittest
-
-try:
-    import mock
-except ImportError: # pragma: no cover
-    from unittest import mock
+from unittest import mock
 
 from certbot.compat import os
 
