@@ -1,10 +1,6 @@
 """Tests for renewal updater interfaces"""
 import unittest
-
-try:
-    import mock
-except ImportError: # pragma: no cover
-    from unittest import mock
+from unittest import mock
 
 from certbot import interfaces
 from certbot._internal import main
