@@ -4,12 +4,7 @@ import logging
 import shutil
 import tempfile
 import unittest
-
-try:
-    import mock
-except ImportError: # pragma: no cover
-    from unittest import mock
-import six
+from unittest import mock
 
 from certbot import errors
 from certbot.compat import os
@@ -19,7 +14,7 @@ from certbot.tests import util as test_util
 class ReverterCheckpointLocalTest(test_util.ConfigTestCase):
     """Test the Reverter Class."""
     def setUp(self):
-        super(ReverterCheckpointLocalTest, self).setUp()
+        super().setUp()
         from certbot.reverter import Reverter
 
         # Disable spurious errors... we are trying to test for them
@@ -49,7 +44,7 @@ class ReverterCheckpointLocalTest(test_util.ConfigTestCase):
         no_change = os.path.join(self.reverter.config.backup_dir, path, "CHANGES_SINCE")
         with open(no_change, "r") as f:
             x = f.read()
-        self.assertTrue("No changes" in x)
+        self.assertIn("No changes", x)
 
     def test_basic_add_to_temp_checkpoint(self):
         # These shouldn't conflict even though they are both named config.txt
@@ -156,7 +151,7 @@ class ReverterCheckpointLocalTest(test_util.ConfigTestCase):
 
         act_coms = get_undo_commands(self.config.temp_checkpoint_dir)
 
-        for a_com, com in six.moves.zip(act_coms, coms):
+        for a_com, com in zip(act_coms, coms):
             self.assertEqual(a_com, com)
 
     def test_bad_register_undo_command(self):
@@ -281,7 +276,7 @@ class ReverterCheckpointLocalTest(test_util.ConfigTestCase):
 class TestFullCheckpointsReverter(test_util.ConfigTestCase):
     """Tests functions having to deal with full checkpoints."""
     def setUp(self):
-        super(TestFullCheckpointsReverter, self).setUp()
+        super().setUp()
         from certbot.reverter import Reverter
         # Disable spurious errors...
         logging.disable(logging.CRITICAL)
@@ -329,8 +324,8 @@ class TestFullCheckpointsReverter(test_util.ConfigTestCase):
         # One dir left... check title
         all_dirs = os.listdir(self.config.backup_dir)
         self.assertEqual(len(all_dirs), 1)
-        self.assertTrue(
-            "First Checkpoint" in get_save_notes(
+        self.assertIn(
+            "First Checkpoint", get_save_notes(
                 os.path.join(self.config.backup_dir, all_dirs[0])))
         # Final rollback
         self.reverter.rollback_checkpoints(1)
