@@ -5,18 +5,13 @@ from importlib import reload as reload_module
 import io
 import sys
 import unittest
+from unittest import mock
 
 from certbot import errors
 from certbot.compat import filesystem
 from certbot.compat import os
 import certbot.tests.util as test_util
 
-try:
-    import mock
-except ImportError: # pragma: no cover
-    from unittest import mock
-
-
 
 class EnvNoSnapForExternalCallsTest(unittest.TestCase):
     """Tests for certbot.util.env_no_snap_for_external_calls."""
@@ -624,5 +619,38 @@ class AtexitRegisterTest(unittest.TestCase):
             atexit_func(*args[1:], **kwargs)
 
 
+class ParseLooseVersionTest(unittest.TestCase):
+    """Test for certbot.util.parse_loose_version.
+
+    These tests are based on the original tests for
+    distutils.version.LooseVersion at
+    https://github.com/python/cpython/blob/v3.10.0/Lib/distutils/tests/test_version.py#L58-L81.
+
+    """
+
+    @classmethod
+    def _call(cls, *args, **kwargs):
+        from certbot.util import parse_loose_version
+        return parse_loose_version(*args, **kwargs)
+
+    def test_less_than(self):
+        comparisons = (('1.5.1', '1.5.2b2'),
+            ('3.4j', '1996.07.12'),
+            ('2g6', '11g'),
+            ('0.960923', '2.2beta29'),
+            ('1.13++', '5.5.kw'))
+        for v1, v2 in comparisons:
+            self.assertLess(self._call(v1), self._call(v2))
+
+    def test_equal(self):
+        self.assertEqual(self._call('8.02'), self._call('8.02'))
+
+    def test_greater_than(self):
+        comparisons = (('161', '3.10a'),
+            ('3.2.pl0', '3.1.1.6'))
+        for v1, v2 in comparisons:
+            self.assertGreater(self._call(v1), self._call(v2))
+
+
 if __name__ == "__main__":
     unittest.main()  # pragma: no cover
